#! /usr/bin/env python

import os
import sys
import MP3Info

def GetTrackInfo(filePath):
	length = 0
	title = ""
	artist = ""
	album = ""
	try:
		f = open(filePath, 'rb')
		i = MP3Info.MP3Info(f)
		length = i.mpeg.total_time
		title = i.title
		artist = i.artist
		album = i.album
	except:
		print "Error extracting length from ", filePath
	
	return length, title, artist, album

def WriteLibraryFolder(f, curPath, rootPath):
	trackCount = 0;

	dirs = []

	for fileName in os.listdir(curPath):
		fullName = os.path.normpath(os.path.join(curPath, fileName))
		fullName = fullName.replace('\\', '/')

		if os.path.isdir(fullName):
			dirs.append(fullName)
			
		else:
			if fullName.lower().endswith('.mp3'):
				# Strip the root and the extension
				shortName = fullName.replace(rootPath, '')
				shortName = shortName.replace('.mp3', '')
				shortName = shortName.replace('.MP3', '')
				shortName = shortName.replace('.Mp3', '')
				shortName = shortName.replace('/', '\\') #remove forward-slashes
				shortName = shortName.replace('\\', '\\\\') 
				
				# Fetch the length of the track in seconds.
				length, title, artist, album = GetTrackInfo(fullName)
				
				# Replace quotes
				if (title):
					title = title.replace('\\', '\\\\') 
					title = title.replace('"', '\\"')
					
				if (artist):
					artist = artist.replace('\\', '\\\\') 
					artist = artist.replace('"', '\\"')
					
				if (album):
					album = album.replace('\\', '\\\\') 
					album = album.replace('"', '\\"')

				if length > 0:
					f.write('   Soundtrack.Library.AddTrack("%s", %s, "%s", "%s", "%s");\n' % (shortName, length, title, artist, album));
					print 'Found %s (%s secs)' % (shortName, length)
					trackCount = trackCount + 1

	for dirName in dirs:
		trackCount = WriteLibraryFolder(f, dirName, rootPath) + trackCount
	
	return trackCount

def main(argv, stdout, environ):
	pathname = os.path.dirname(sys.argv[0])
	workingPath = os.path.abspath(pathname)	
	workingPath = workingPath.replace('\\', '/')
	
	mp3Path = workingPath + '/Music/'
	scriptFilePath = workingPath + '/MyTracks.lua'

	

	if not os.path.isdir(mp3Path):
		print 'ERROR : Cannot find the folder ' + mp3Path
		print 'Soundtrack needs to have your music under Interface/AddOns/Soundtrack/Music'
		raw_input("Press ENTER to continue...")
		getch();

	trackCount = 0;
	f = open(scriptFilePath, 'w')
	f.write('function Soundtrack_LoadMyTracks()\n')
    
	print 'Scanning ' + mp3Path

	trackCount = WriteLibraryFolder(f, mp3Path, mp3Path) + trackCount

	f.write('end\n')
	f.close()

	print ' '
	print 'Finished writing MyTracks.lua.'
	print 'Found %s track(s).' % trackCount
	print 'The tracks should now be available within the game!'
	print ' '
	raw_input("Press ENTER to continue...")


if __name__ == "__main__":
  main(sys.argv, sys.stdout, os.environ)

