﻿-- Globals


Soundtrack_EventTabs = {
	"Battle", 
	"Boss",
	"Zone", 
	"Dance",
	"Misc",
	"Custom",
	"Playlists"
};

Soundtrack_Events = {}
Soundtrack_CustomEvents = {}
Soundtrack_MiscEvents = {}
Soundtrack_SortedEvents = {}

Soundtrack_Tracks = nil
Soundtrack_SortedTracks = {}

SOUNDTRACK_DEFAULT_EVENTS = 
{
	["Misc"] = 
	{

	},
	
	["Custom"] =
	{
	},
	
	["Playlists"] = 
	{
	
	}
};

-- Support for myAddon
Soundtrack_Details = {
	name = "Soundtrack",
	version = "0.1",
	releaseDate = "January 11, 2006",
	author = "Anthony Brien",
	email = "anthony.brien@gmail.com",
	website = "http://abrien.blogspot.com",
	category = MYADDONS_CATEGORY_AUDIO,
	optionsFrame = "SoundtrackFrame"
};

Soundtrack_Help = {};
Soundtrack_Help[1] = "Help Page1 line1\nline2\nline3...";
Soundtrack_Help[2] = "Help Page2 line1\nline2\nline3...";


local function OnAddonLoaded()
	-- Register the addon in myAddOns
	if(myAddOnsFrame_Register) then
		myAddOnsFrame_Register(Soundtrack_Details, Soundtrack_Help);
	end
end


function Soundtrack.IsNullOrEmpty(text)
    return text == nil or text == ""
end


-- Removes obsolete tracks from events.
local function PurgeEventsFromTable(eventTableName)
	local eventTable = Soundtrack.Events.GetTable(eventTableName);
	
	for k, v in pairs(eventTable) do
		local tracksToRemove = {};
		
		-- Find tracks to remove
		local i;
		for i=1, getn(v.tracks), 1 do
			if (Soundtrack_Tracks[v.tracks[i]] == nil) then
				Soundtrack.Message("Removed obsolete track "..v.tracks[i]);
				table.insert(tracksToRemove, v.tracks[i]);
			end
		end
		
		-- Remove tracks
		local j;
		for j=1, getn(tracksToRemove), 1 do
			Soundtrack.Events.Remove(eventTableName, k, tracksToRemove[j]);
		end

	end
	
end

-- Removes obsolete tracks from event assignments
local function PurgeEvents()
	local i;
	for i = 1, getn(Soundtrack_EventTabs), 1 do
		PurgeEventsFromTable(Soundtrack_EventTabs[i]);
	end

    
end

local function MigrateOldTables()
	
	local k,v,i;
	if (Soundtrack_ZoneEvents) then 
		for k,v in pairs(Soundtrack_ZoneEvents) do
			for i=1, getn(v.tracks), 1 do
				Soundtrack.AssignTrack(k, v.tracks[i]);
			end
		end
	end
	Soundtrack_ZoneEvents = {};
	
	if (Soundtrack_BattleEvents) then 
		for k,v in pairs(Soundtrack_BattleEvents) do
			for i=1, getn(v.tracks), 1 do
				Soundtrack.AssignTrack(k, v.tracks[i]);
			end
		end
	end
	Soundtrack_BattleEvents = {};
	
	if (Soundtrack_DanceEvents) then 
		for k,v in pairs(Soundtrack_DanceEvents) do
			for i=1, getn(v.tracks), 1 do
				Soundtrack.AssignTrack(k, v.tracks[i]);
			end
		end
	end
	Soundtrack_DanceEvents = {};
end

local function InitDefaultSettings()
	if (Soundtrack.Settings.MinimapIconPos == nil) then
		Soundtrack.Settings.MinimapIconPos = 45;
	end
	
	if (Soundtrack.Settings.EnableMinimapButton == nil) then
		Soundtrack.Settings.EnableMinimapButton = true;
	end
	
	if (Soundtrack.Settings.ShowTrackInformation == nil) then
		Soundtrack.Settings.ShowTrackInformation = true;
	end
	
	if Soundtrack.Settings.ShowDefaultMusic == nil then
	    Soundtrack.Settings.ShowDefaultMusic = false
	end
	
	if (Soundtrack.Settings.ShowEventStack == nil) then
		Soundtrack.Settings.ShowEventStack = false;
	end
	
	if (Soundtrack.Settings.ShowPlaybackControls == nil) then
		Soundtrack.Settings.ShowPlaybackControls = false;
	end
	
	if (Soundtrack.Settings.EnableBattleMusic == nil) then
		Soundtrack.Settings.EnableBattleMusic = true;
	end
	
	if Soundtrack.Settings.EnableZoneMusic == nil then
		Soundtrack.Settings.EnableZoneMusic = true
	end
	
	if (Soundtrack.Settings.EnableMiscMusic == nil) then
		Soundtrack.Settings.EnableMiscMusic = true;
	end
	
	if (Soundtrack.Settings.EnableCustomMusic == nil) then
		Soundtrack.Settings.EnableCustomMusic = true;
	end
	
	if (Soundtrack.Settings.Debug == nil) then
		Soundtrack.Settings.Debug = false;
	end
	
	if (Soundtrack.Settings.BattleCooldown == nil) then
		Soundtrack.Settings.BattleCooldown = 0;
	end
	
	if (Soundtrack.Settings.Silence == nil) then
		Soundtrack.Settings.Silence = 5;
	end
	
	if (Soundtrack.Settings.AutoAddZones == nil) then
	    Soundtrack.Settings.AutoAddZones = false
	end
	
	if Soundtrack.Settings.EscalateBattleMusic == nil then
	    Soundtrack.Settings.EscalateBattleMusic = true
	end
	
	if Soundtrack.Settings.trackSortingCriteria == nil then
	    Soundtrack.Settings.trackSortingCriteria = "filePath"
	end
end

local function GetPathFileNameRecursive(filePath)
    local index1 = string.find(filePath, "/")
    if index1 == nil then
        return filePath
    else
        return Soundtrack.GetPathFileName(string.sub(filePath, index1 + 1))
    end
end


-- Strips a path string from the path and returns the file name at the end.
-- Used to strip parent events or parent folders from event and track names.
function Soundtrack.GetPathFileName(filePath)
    local temp = string.gsub(filePath, "\\", "/") 
    return GetPathFileNameRecursive(temp)
end

function Soundtrack.AddEvent(tableName, eventName, _priority, _continuous)
  
    if Soundtrack.IsNullOrEmpty(tableName) then
        Soundtrack.Error("AddEvent: Nil table")
        return
    end
    
    if Soundtrack.IsNullOrEmpty(eventName) then
        Soundtrack.Error("AddEvent: Nil event")
        return
    end
    
    local eventTable = Soundtrack.Events.GetTable(tableName)
    
    if eventTable == nil then
	    Soundtrack.Error("AddEvent: Cannot find table : " .. tableName)
	    return
	end
	
	local event = eventTable[eventName]
	if event ~= nil then
	    event.priority = _priority
	    event.continuous = _continuous
	   
	    return -- event is already registered
	end
	
	Soundtrack.TraceEvents("AddEvent: " .. tableName .. ": " .. eventName);
    
	eventTable[eventName] = 
	{ 
	    tracks = {}, 
	    lastTrackIndex = 0, 
	    random = true, 
	    priority = _priority, 
	    continuous = _continuous 
	}  
	
	-- Because I cant figure out how to sort the hashtable...
	Soundtrack_SortEvents(tableName);
end

function Soundtrack.AssignTrack(eventName, trackName)
    if Soundtrack.IsNullOrEmpty(eventName) or Soundtrack.IsNullOrEmpty(trackName) then
        debugEvents("AddEvent: Invalid table or event name")
        return
    end

    local tableName = Soundtrack.GetTableFromEvent(eventName)
	local eventTable = Soundtrack.Events.GetTable(tableName)
	
	if eventTable == nil then
	    debugEvents("Cannot find table : " .. tableName)
	    return
	end
	
	table.insert(eventTable[eventName].tracks, trackName);
end

local function OnVariablesLoaded()

	if (Soundtrack_Settings == nil) then
		Soundtrack_Settings = {};
	end
	Soundtrack.Settings = Soundtrack_Settings;
	
	Soundtrack.Timers.AddTimer("InitDebugChatFrame", 2, Soundtrack.Util.InitDebugChatFrame);
	InitDefaultSettings();

	Soundtrack_Tracks = {};
	
	-- Load tracks in generated script if available
	if Soundtrack_LoadDefaultTracks ~= nil then
        Soundtrack_LoadDefaultTracks()
    end

	if (Soundtrack_LoadMyTracks ~= nil) then
		Soundtrack_LoadMyTracks();
	else
		error("Cannot load MyTracks.lua");
	end	

	-- Create events table
	local i;
	for i=1, getn(Soundtrack_EventTabs), 1 do
		local eventTabName = Soundtrack_EventTabs[i];
		if (Soundtrack_Events[eventTabName] == nil) then
			Soundtrack.TraceEvents("Creating table "..eventTabName);
			Soundtrack_Events[eventTabName] = {};
		end
	end
	
	MigrateOldTables();
	
	Soundtrack.DanceEvents.Initialize()
	Soundtrack.ZoneEvents.Initialize()
    Soundtrack.BattleEvents.Initialize()
    Soundtrack.MountEvents.Initalize()
    Soundtrack_MiscEvents = {}

	
	Soundtrack.CustomEvents.Initialize();
		
	-- Remove obsolete predefined events
	PurgeEvents();

	-- sort all the tables
	for i=1, getn(Soundtrack_EventTabs), 1 do
	    Soundtrack_SortEvents(Soundtrack_EventTabs[i])
	end
	
	Soundtrack.SortTracks();
	
	local numTracks = getn(Soundtrack_SortedTracks);
	Soundtrack.Message("Loaded with " .. numTracks.." track(s) in library.");
	Soundtrack.Trace("Num battle events "..getn(Soundtrack_SortedEvents["Battle"]));
	Soundtrack.Trace("Num zone events "..getn(Soundtrack_SortedEvents["Zone"]));
	Soundtrack.Trace("Num dance events "..getn(Soundtrack_SortedEvents["Dance"]));
	Soundtrack.Trace("Num misc events "..getn(Soundtrack_SortedEvents["Misc"]));
	Soundtrack.Trace("Num custom events "..getn(Soundtrack_SortedEvents["Custom"]));
	Soundtrack.Trace("Num playlists "..getn(Soundtrack_SortedEvents["Playlists"]));
	
	SoundtrackFrame_RefreshPlaybackControls();
end

function Soundtrack.GetTableFromEvent(eventName)
    
    for i=1, getn(Soundtrack_EventTabs) do
        local eventTable = Soundtrack.Events.GetTable(Soundtrack_EventTabs[i])
        if eventTable ~= nil and eventTable[eventName] ~= nil then
            return Soundtrack_EventTabs[i]
        end
    end
    
    return nil
end

function Soundtrack.GetEvent(tableName, eventName)
    if tableName == nil or eventName == nil then
        return nil
    end

    local eventTable = Soundtrack.Events.GetTable(tableName)
	if eventTable[eventName] ~= nil then
	    return eventTable[eventName]
	else
	    return nil
	end
end

function Soundtrack.GetEventByName(eventName)
    local tableName = Soundtrack.GetTableFromEvent(eventName)
	if tableName == nil then
	    return nil
	end
		
	if (tableName == nil) then
		return nil;
	end
	if (eventName == nil) then
		return nil;
	end

    local eventTable = Soundtrack.Events.GetTable(tableName)
    return eventTable[eventName]
end

-- Todo Replace force restart with another method to "reset" an event or clear its history
function Soundtrack.PlayEvent(tableName, eventName, forceRestart)

	if tableName == nil then
		Soundtrack.Error("PlayEvent: Invalid table name")
	    return
	end

	if eventName == nil then
		Soundtrack.Error("PlayEvent: Invalid event name")
		return
	end

    local eventTable = Soundtrack.Events.GetTable(tableName)
    local event = eventTable[eventName]
    
    if event == nil then
        return
    end
	
	local playOnceText;
	if event.continuous then
		playOnceText = "Loop";
	else 
		playOnceText = "Once";
	end
	
	
	local priorityText
	if event.priority ~= nil then
	    priorityText = "Priority: " .. event.priority
	else
	    priorityText = "Priority: Unset"
	end
	
	local sfxText = ""
	if event.soundEffect then
	    sfxText = " (SFX)"
	end
	
	Soundtrack.TraceEvents("PlayEvent("..tableName..", ".. Soundtrack.GetPathFileName(eventName) ..", ".. priorityText ..") "..playOnceText .. sfxText);	

	if offset == nil then 
		offset = 1; 
	end

	-- Add event on the stack	
	if event.priority == nil then
	    Soundtrack.Error("Cannot play event " .. eventName .. ". It has no priority!")
	elseif event.soundEffect then
	    -- Sound effects are never added to the stack
	    PlayRandomTrackByTable(tableName, eventName, offset)
	else
	    Soundtrack.Events.Stack[event.priority].tableName = tableName;
	    Soundtrack.Events.Stack[event.priority].eventName = eventName;
	    Soundtrack.Events.Stack[event.priority].offset = offset;
	    Soundtrack.Events.OnStackChanged(forceRestart);
	end
end

-- Call this to stop music. Makes sure it only stops music Soundtrack has started.
function Soundtrack.StopEventAtLevel(stackLevel)	
	--verifyStackLevel(stackLevel);

    if stackLevel == nil then
        return
    end
	
	Soundtrack.TraceEvents("StopEvent("..stackLevel..")");
	
	-- Remove the event from the stack.
	Soundtrack.Events.Stack[stackLevel].eventName = nil;
	Soundtrack.Events.Stack[stackLevel].tableName = nil;
	Soundtrack.Events.Stack[stackLevel].offset = 0;
	Soundtrack.Events.OnStackChanged();
end

function Soundtrack.StopEvent(tableName, eventName)	
    local event = Soundtrack.GetEvent(tableName, eventName)
    if event ~= nil then
        Soundtrack.StopEventAtLevel(event.priority)
    end
end

function Soundtrack.ShowTip(tipTitle, tipText)
    GameTooltip_AddNewbieTip(tipTitle, 1.0, 1.0, 1.0, tipText, 1);
end

function Soundtrack_SortEvents(eventTableName)

	-- Only sort if the table is the zones.

	Soundtrack_SortedEvents[eventTableName] = {};
	local k,v;
	for k,v in pairs(Soundtrack_Events[eventTableName]) do
	    if k ~= "Preview" then
		    table.insert(Soundtrack_SortedEvents[eventTableName], k);
		end
	end
	
	-- Only sort is user do not wish to bypass
    table.sort(Soundtrack_SortedEvents[eventTableName]);
	
	SoundtrackFrame_RefreshEvents();
end

local function CompareTracksByAlbum(i1, i2)
	return Soundtrack_Tracks[i1].album < Soundtrack_Tracks[i2].album
end

local function CompareTracksByArtist(i1, i2)
	return Soundtrack_Tracks[i1].artist < Soundtrack_Tracks[i2].artist
end

local function CompareTracksByTitle(i1, i2)
	return Soundtrack_Tracks[i1].title < Soundtrack_Tracks[i2].title
end

local function CompareTracksByFileName(i1, i2)
	return Soundtrack.GetPathFileName(i1) < Soundtrack.GetPathFileName(i2)
end

local function MatchTrack(trackName, track, filter)

    local lowerFilter = string.lower(filter)
    local index = string.find(string.lower(trackName), lowerFilter)
    if index ~= nil then
        return true
    end
    
    index = string.find(string.lower(track.artist), lowerFilter)
    if index ~= nil then
        return true
    end
    
    index = string.find(string.lower(track.album), lowerFilter)
    if index ~= nil then
        return true
    end
    
    index = string.find(string.lower(track.title), lowerFilter)
    if index ~= nil then
        return true
    end
    
    return nil
end

function Soundtrack.Message(text)
    Soundtrack.Util.ChatPrint("Soundtrack: " .. text)
end

function Soundtrack.Warning(text)
    Soundtrack.Util.DebugPrint("Soundtrack: Warning: " .. text, 0.75, 0.75, 0.0)
end

function Soundtrack.Error(text)
    Soundtrack.Util.ChatPrint("Soundtrack: Error: " .. text, 1, 0.25, 0.0)
end

function Soundtrack.Trace(text)
    Soundtrack.Util.DebugPrint("[Trace]: " .. text)
end

function Soundtrack.TraceLibrary(text)
    Soundtrack.Util.DebugPrint("[Library]: " .. text, 0.25, 0.25, 1.0)
end

function Soundtrack.TraceBattle(text)
    Soundtrack.Util.DebugPrint("[Battle]: " .. text, 1.0, 0.0, 0.0)
end

function Soundtrack.TraceZones(text)
    Soundtrack.Util.DebugPrint("[Zones]: " .. text, 0.0, 0.50, 0.0)
end

function Soundtrack.TraceFrame(text)
    Soundtrack.Util.DebugPrint("[Frame]: " .. text, 0.75, 0.75, 0.0)
end

function Soundtrack.TraceEvents(text)
    Soundtrack.Util.DebugPrint("[Events]: " .. text, 0.0, 1.0, 1.0)
end



-- Sorts the list of tracks using the passed criteria, or uses
-- the last used sorting criteria
function Soundtrack.SortTracks(sortCriteria)

	if sortCriteria ~= nil then
		Soundtrack.lastSortCriteria = sortCriteria
	elseif sortCriteria == nil then
	    sortCriteria = Soundtrack.lastSortCriteria
	end
	
	Soundtrack_SortedTracks = {};
	local k,v;
	for k,v in pairs(Soundtrack_Tracks) do
	    if not v.defaultTrack or Soundtrack.Settings.ShowDefaultMusic then
	        if Soundtrack.trackFilter == nil then 
		        table.insert(Soundtrack_SortedTracks, k);
	        elseif MatchTrack(k, v, Soundtrack.trackFilter) then
	            table.insert(Soundtrack_SortedTracks, k);
	        end
	    end
	end
	
	if sortCriteria == "album" then
		table.sort(Soundtrack_SortedTracks, CompareTracksByAlbum)
	elseif sortCriteria == "artist" then
		table.sort(Soundtrack_SortedTracks, CompareTracksByArtist)
	elseif sortCriteria == "filePath" then
	    table.sort(Soundtrack_SortedTracks)
	elseif sortCriteria == "fileName" then
	    table.sort(Soundtrack_SortedTracks, CompareTracksByFileName)
	elseif sortCriteria == "title" then
	    table.sort(Soundtrack_SortedTracks, CompareTracksByTitle)
	end
	
	SoundtrackFrame_RefreshTracks();
end


local function OnOpenOptionsFrame()
	SoundtrackFrame:Show();
end

local function ToggleDebugMode()
	Soundtrack.Settings.Debug = not Soundtrack.Settings.Debug;
	Soundtrack.Util.InitDebugChatFrame();
end

local delayTime = 0
local updateTime = 1

function Soundtrack_OnUpdate(deltaT)
	
	local currentTime = GetTime()
    if currentTime >= delayTime then
        delayTime = currentTime + updateTime
	    
	    Soundtrack.Timers.OnUpdate(deltaT);
	    
	    Soundtrack.MountEvents.OnUpdate(deltaT);
	    Soundtrack.CustomEvents.OnUpdate(deltaT);
	    Soundtrack.BattleEvents.OnUpdate();
	    SoundtrackFrame_RefreshTrackProgress()
	end
end

function Soundtrack_OnLoad()

	-- Register command
	SLASH_SOUNDTRACK1 = "/soundtrack";
	SlashCmdList["SOUNDTRACK"] = OnOpenOptionsFrame;
	
	SLASH_SOUNDTRACKDEBUG1 = "/soundtrackdebug";
	SlashCmdList["SOUNDTRACKDEBUG"] = ToggleDebugMode;

	-- Register events
	this:RegisterEvent("VARIABLES_LOADED"); 
	this:RegisterEvent("ADDON_LOADED");

	Soundtrack.BattleEvents.OnLoad();
	Soundtrack.ZoneEvents.OnLoad();
	Soundtrack.DanceEvents.OnLoad();
	Soundtrack.MountEvents.OnLoad();
end



function Soundtrack_OnEvent(arg1, arg2)

	if (event == "VARIABLES_LOADED") then
		OnVariablesLoaded();
	elseif (event == "ADDON_LOADED") then
		OnAddonLoaded();	
	end
	
	Soundtrack.ZoneEvents.OnEvent(arg1, arg2);
	Soundtrack.BattleEvents.OnEvent(arg1, arg2);
	Soundtrack.DanceEvents.OnEvent(arg1, arg2);
	Soundtrack.CustomEvents.OnEvent(arg1, arg2);
end