--[[
	Soundtrack addon for World of Warcraft

	Soundtrack library functions.
	Functions that manage the users list of available music tracks.
]]

-- Remove track popup
StaticPopupDialogs["SOUNDTRACK_REMOVETRACK"] = {
	text = [[Do you want to remove this track from your library?]],
	button1 = "OK",
	button2 = "Cancel",
	OnAccept = function() 
		Soundtrack.Library.RemoveTrack(SoundtrackFrame_SelectedTrack); 
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
}

Soundtrack.Library = {

	CurrentlyPlayingTrack = nil;

	AddTrack = function(trackName, _length, _title, _artist, _album)
		-- Replace old tracks
		if (Soundtrack_Tracks[trackName] == nil) then
			Soundtrack_Tracks[trackName] = nil;
		end
					
		Soundtrack_Tracks[trackName] = { length = _length, title = _title, artist = _artist, album = _album };
		Soundtrack.SortTracks();
	end;
	
	AddDefaultTrack = function(trackName, _length, _title, _artist, _album)
	    -- Replace old tracks
		if (Soundtrack_Tracks[trackName] == nil) then
			Soundtrack_Tracks[trackName] = nil;
		end
					
		Soundtrack_Tracks[trackName] = { length = _length, title = _title, artist = _artist, album = _album, defaultTrack = true };
		Soundtrack.SortTracks();
	end;

	PauseMusic = function()
		StopMusic();
		Soundtrack.Library.CurrentlyPlayingTrack = "None";
		Soundtrack.TraceLibrary("Playing empty track.");
		PlayMusic("Interface\\AddOns\\Soundtrack\\EmptyTrack.mp3");
		SoundtrackFrame_TouchTracks();
	end;
	
	PlayTrack = function(trackName, soundEffect)
		Soundtrack.TraceLibrary("PlayTrack(".. Soundtrack.GetPathFileName(trackName) ..")");
		
		-- Check if the track is valid
		if (Soundtrack_Tracks[trackName] == nil) then
			return;
		end
		
		-- Check if that track is already playing
		if (trackName == Soundtrack.Library.CurrentlyPlayingTrack) then
			return;
		end
		
	    local trackInfo = Soundtrack_Tracks[trackName];
        
        -- TODO : Change DefaultScore name
		local fileName
		if trackInfo.defaultTrack then
		    fileName = "Sound\\Music\\" .. trackName .. ".mp3";
		else
		    fileName = "Interface\\AddOns\\Soundtrack\\Music\\"..trackName..".mp3";
		end	    
		
		-- Everything ok, play the track
		if soundEffect == nil or not soundEffect then
		    Soundtrack.Library.CurrentlyPlayingTrack = trackName;
	        SetNowPlayingText(trackInfo.title, trackInfo.artist, trackInfo.album);
    		PlayMusic(fileName);
        else
            PlaySoundFile(fileName) -- sound effect. play the music overlapping other music
        end
		
		-- Update the UI if its opened
		SoundtrackFrame_TouchTracks();
	end;
	
	StopTrack = function()
		Soundtrack.TraceLibrary("StopTrack()");
		Soundtrack.Library.CurrentlyPlayingTrack = nil;
		StopMusic();
		SoundtrackFrame_TouchTracks();
	end;
	
	-- Removes a track from the library.
	RemoveTrackWithConfirmation = function()		
		-- Confirmation
		StaticPopup_Show("SOUNDTRACK_REMOVETRACK");
	end;
	
	RemoveTrack = function(trackName)
		if (trackName ~= nil) then
			Soundtrack_Tracks[trackName] = nil;
			Soundtrack.SortTracks();
			
			-- Remove the track from any event that assigns it
			
			local key, value, eventTab;
			for eventTab = 1, getn(Soundtrack_EventTabs), 1 do
				for key,value in Soundtrack_Events[Soundtrack_EventTabs[eventTab]] do 
					for i=1, getn(value.tracks), 1 do
						if (value.tracks[i] == trackName) then
							Soundtrack.TraceLibrary("Removed assigned track "..trackName);
							table.remove(value.tracks, i);
							break
						end
					end
				end
			end
			
			-- Refresh assigned counts
			SoundtrackFrame_RefreshEvents();
		end
	end;
};